# -*- coding: utf-8 -*-
"Implémentation objet de listes mutables"

class Cellule:
    def __init__(self, valeur, suivant):
        self._valeur = valeur
        self._suivant = suivant
        
class Liste:
    def __init__(self, c, longueur=0):
        self.cellule = c
        self._longueur = longueur
        
    def valeur(self):
        return self.cellule._valeur
    
    def suite(self):
        return Liste(self.cellule._suivant, self._longueur-1)

    def ajoute(self, valeur):
        """ajoute la valeur en tête de liste, en modifiant
        l'objet liste (listes mutables)
        """
        self.cellule = Cellule(valeur, self.cellule)
        self._longueur += 1

    """
    Si on veut des listes immuables, on utilise la fonction
    ajoute ci-dessous, qui ne modifie pas l'objet actif mais
    crée et renvoie un autre objet :
        
    def ajoute(self, valeur):
        return Liste(Cellule(valeur, self.cellule))
    """
    
    def est_vide(self):
        return self.cellule is None
    
    def longueur(self):
        return self._longueur
    
    def __repr__(self):
        if self.est_vide():
            return "[]"
        suivant = self.suite()
        valeur = self.valeur()
        res = "[" + str(valeur) + ", "
        while not suivant.est_vide():
            res = res + str(suivant.valeur()) + ", "
            suivant = suivant.suite()
        return res[:-2] + "]"

    def min(self):
        "Version fonctionnelle, utilise la fonction min de Python"
        if self.suite().est_vide():
            return self.valeur()
        else:
            return min(self.valeur(), self.suite().min())

    def max(self):
        if self.suite().est_vide():
            return self.valeur()
        else:
            return max(self.valeur(), self.suite().max())

    
def liste_vide():# fonction Ã©crite en dehors de la classe
    return Liste(None)

if __name__ == '__main__':
    lst = liste_vide()
    lst.ajoute(1)
    print(lst.valeur())
    lst.ajoute(2)
    print(lst.valeur())
    lst.ajoute(3)
    print(lst.valeur())
